#include <iostream>
#include <string>
#include <unistd.h>
#include <sys/times.h>
#include "types.h"

using namespace std;


class Timer {
public:
	Timer(): wallTime(0),userTime(0),systemTime(0) {
		ticks_per_second = sysconf(_SC_CLK_TCK);
	};
	~Timer() {};
	
	void start();
	void stop();
	void report(const char* name);
	double secondsPassed(clock_t start,clock_t stop);

	clock_t ticks_per_second;
	clock_t start_wtime;
	clock_t	start_utime;
	clock_t start_stime;
	double wallTime;
	double userTime;
	double systemTime;
};

inline void
Timer::start()
{
	struct tms buf;
	start_wtime = times(&buf);
	start_utime = buf.tms_utime;
	start_stime = buf.tms_stime;
}

inline void
Timer::stop()
{
	struct tms buf;
	clock_t stop_wtime = times(&buf);
	wallTime += secondsPassed(start_wtime,stop_wtime);
	userTime += secondsPassed(start_utime,buf.tms_utime);
	systemTime += secondsPassed(start_stime,buf.tms_stime);	
}

void
Timer::report(const char* name)
 {   
	cout << "Timer " << name << ": " << userTime << "u " << systemTime << "s "
		 << wallTime << "w" << endl;
	userTime = systemTime = wallTime = 0; 
}

inline double Timer::secondsPassed(clock_t start,clock_t stop)
{
	if (start <= stop) {
		return  (static_cast<double>(stop) - start)/ticks_per_second;
	} else {
	    cerr << "Timer::start > Timer::stop: " << start << " " << stop << endl;
		return ((static_cast<double>(maxSignedValue(bitSizeof(clock_t))) - start)
			 + start)/ticks_per_second;
	}
}


   
