//==============================================================================
//      gsort.h  -  sort goemetric objects
//                           -------------------
//  begin                : |DATE|
//  copyright            : (C) |YEAR| by |AUTHOR|
//  email                : |EMAIL|
//==============================================================================

#ifndef GSORT_H
#define GSORT_H

#include <shape.h>

#define gsortTemplate template <typename baseType, typename coordinate_T, \
								typename geom_T, typename shapeBlock_T>
#define gsortTmpltArgs	baseType,coordinate_T,geom_T,shapeBlock_T

gsortTemplate  
class gsortShapeNode {
public:
	gsortShapeNode(): _shapes(0) {}
	
	//===================================================================
	// Accessors
	//===================================================================
	bool isFull() { return _shapes && _shapes->isFull(); }
	
	//===================================================================
	// Manipulators
	//===================================================================
private:
	shapeBlock_T* _shapes;
};


gsortTemplate
class gsortTreeNode: public gsortShapeNode<gsortTmpltArgs> {
public:
	// Used for initial construction of gsortTreeNode
	gsortTreeNode(bool splitX,bool reduce):
	 	_splitInX(splitX),_cross(false),_reduce(reduce),_grtr(0),_less(0) {}
	
	// Used when constructing a gsortTreeNodeWithCross 
	gsortTreeNode(const gsortTreeNode& ntree):
	 	_splitInX(ntree.splitX()),_cross(true),_reduce(ntree.reduce()),
		_grtr(ntree.grtr()),_less(ntree.less()) {}
	
		
	//=========================================================================
	// Accessors
	//=========================================================================
	bool splitInX() { return _splitInX; }
	bool cross()	{ return _cross; }
	bool reduce() { return _reduce; }
	bool isSplit() { return grtr() || less(); }
	gsortTreeNode<gsortTmpltArgs>* grtr() { return _grtr; }
	gsortTreeNode<gsortTmpltArgs>* less() { return _less; }
	
	//=========================================================================
	// Manipulators
	//=========================================================================
	void grtr(gsortTreeNode<gsortTmpltArgs>* node)
	{
		_grtr(node);
	}
	void less(gsortTreeNode<gsortTmpltArgs>* node)
	{
		_less(node);
	}
	// Allocate space at or below this node in the tree.
	void* allocate(gsortTreeNode<gsortTmpltArgs>** parentPtr,
			const Box& logical,const Box& bbox, int reclen);
	// Allocate space at this node in the tree.
	void* allocate(const Box& bbox, int reclen);
private:
    bool 		_splitInX;		// 1: split in X; 0: split in Y
	bool		_cross;			// CrossNodeExists
	bool 		_reduce;		// Next level of tree reduces coordinate_T	
	gsortTreeNode<gsortTmpltArgs>* _grtr;
	gsortTreeNode<gsortTmpltArgs>* _less;
};

           

gsortTemplate
class gsortCrossNode: public gsortShapeNode<gsortTmpltArgs> {
public:
		gsortCrossNode():
			_above(0),_below(0)	{}
private:
	gsortCrossNode<gsortTmpltArgs>* _above;
	gsortCrossNode<gsortTmpltArgs>* _below;
};

gsortTemplate
class gsortTreeNodeWithCross: public gsortTreeNode<gsortTmpltArgs> {
public:
	gsortTreeNodeWithCross(const gsortTreeNode<gsortTmpltArgs>& v):
		gsortTreeNode<gsortTmpltArgs>(v),_above(0),_below(0) {}	
private:
	gsortCrossNode<gsortTmpltArgs>* _above;
	gsortCrossNode<gsortTmpltArgs>* _below;
};



gsortTemplate
class gsortTree {
public:
	gsortTree(): _top(0) {}
	//======================================================================
	// Accessor
	//======================================================================
	gsortTreeNode<gsortTmpltArgs>* top() { return _top; }
	Box& logical() { return _logical; }
	Box& data() { return _data; }
	
	//======================================================================
	// Modifier
	//======================================================================
	// Allocate space in the tree for a varGeom with bbox and reclen.
	void* allocate(Box bbox,int reclen);
	
private:
	gsortTreeNode<gsortTmpltArgs>* _top;
	Box		_logical;	// Bounding box of top node of the tree.
	Box		_data;		// Bounding box of data stored in tree.
	
};

#endif
